﻿package  
{
	import Box2D.Collision.Shapes.b2PolygonDef;
	import Box2D.Collision.Shapes.b2PolygonShape;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.Joints.b2MouseJoint;
	import Box2D.Dynamics.Joints.b2MouseJointDef;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	

	public class JointBox
	{
		
		static internal const timeStep:Number = 1 / 24;

		static internal var container:Sprite;
		static internal var world:b2World;		
		static internal var size:Point;
		static internal var force:Number;
		static internal var density:Number;
		static internal var restitution:Number;
		static internal var friction:Number;

		internal var body:b2Body;
		private var target:b2Body;
		internal var joint:b2MouseJoint;
		internal var joint2:b2MouseJoint;
		
		static internal function initParam(wld:b2World, ctr:Sprite):void {

			container = ctr;
			world = wld;		
			size = new Point(0.1, 0.05);
			force = 50;
			density = 10;
			restitution = 0.8;
			friction = 0.1;
		}
		
		public function JointBox(tgt:b2Body, x:int, y:int)
		{
			body = world.CreateBody(setBody(x, y));
			body.CreateShape(setShape());
			body.SetMassFromShapes();

			target = tgt;
			setJoint();
			
			container.addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		internal function onEnterFrame(event:Event):void {
			joint.SetTarget(body.GetPosition());
			joint2.SetTarget(target.GetPosition());
		}
		
		private function setBody(x:int, y:int):b2BodyDef {

			var boxBodyDef:b2BodyDef = new b2BodyDef();
			boxBodyDef.position.Set(x, y);
			
			return boxBodyDef;
		}
		
		private function setShape():b2PolygonDef {
			
			var boxShapeDef:b2PolygonDef = new b2PolygonDef();

			boxShapeDef.SetAsOrientedBox(size.x, size.y);
			boxShapeDef.density = density;
			boxShapeDef.restitution = restitution;
			boxShapeDef.friction = friction;

			return boxShapeDef;
		}
		
		internal function setJoint():void {

			joint = b2MouseJoint(world.CreateJoint(getMouseJointDef(target)));						
			joint2 = b2MouseJoint(world.CreateJoint(getMouseJointDef(body)));			
		}
		
		private function getMouseJointDef(obj:b2Body):b2MouseJointDef {

			var mouseJointDef:b2MouseJointDef = new b2MouseJointDef(); 

			mouseJointDef.body1 = world.GetGroundBody();
			mouseJointDef.body2 = obj;
			mouseJointDef.target = obj.GetWorldCenter();             
			mouseJointDef.maxForce = force;			
			mouseJointDef.timeStep = timeStep; 			

			return mouseJointDef;
		}

		
	}
	
}